/***************************************************************************
                          gamefield.h  -  description
                             -------------------
    begin                : Tue Mar 28 2000
    copyright            : (C) 2000 by Frank von Daak
    email                : frank.fun@kpage.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GAMEFIELD_H
#define GAMEFIELD_H

#include <qstring.h>
#include <qwidget.h>
#include <qlist.h>

class QCanvas;
class QCanvasText;
class QCanvasSprite;
class QCanvasPixmapArray;

class MyQCanvasView;

/**
  *@author Frank von Daak
  */

class GameField : public QWidget  {
   Q_OBJECT
public:
	GameField(QWidget *parent=0, const char *name=0);
	~GameField();

	void showText( QString text, const QColor &color );
	void newGame();
	void reset();
	void setGnatSpeed( int s ) { gnatSpeed = s; }
	void addGnats( int num );
	void addMag( int num );
	void addTux( int num );
	void setLevel(int);
	void showPause(bool);

	MyQCanvasView *view;
	QCanvasText *textSprite;
	QCanvas *field;

	bool started, sound;
	int shots;

	void addBullets(int);
	void removeBullets(int);

protected:
	void readSprites();
	virtual void timerEvent( QTimerEvent * );


private:
	QList<QCanvasSprite> bullets;
	QCanvasPixmapArray *bulletImg;

	QTimer* bulletTimer;

	QList<QCanvasSprite> gnats;
	QCanvasPixmapArray *leftMImg;
	QCanvasPixmapArray *rightMImg;

	QCanvasPixmapArray *leftBImg;
	QCanvasPixmapArray *rightBImg;

	QList<QCanvasSprite> dgnats;
	QCanvasPixmapArray *GnatKillImg;

	QList<QCanvasSprite> mags;
	QCanvasPixmapArray *magImg;

	QList<QCanvasSprite> tuxs;
	QCanvasPixmapArray *tuxImg;
	QList<QCanvasSprite> dtuxs;
	QCanvasPixmapArray *TuxKillImg;


	QCanvasText *pauseTextSprite;

	const char *IMG_BACKGROUND, *L_MUEKE, *R_MUEKE, *L_MMASK, *R_MMASK,
					*L_BUCK, *R_BUCK, *R_BMASK, *L_BMASK;
	int textDy, fc, ft, shootWait, L_MFRAMES, R_MFRAMES, L_BFRAMES,
					R_BFRAMES, rnd;
	double gnatSpeed;

	void showDeadGnat(int x, int y);
	void showDeadTux(int x, int y);
	void removeDGnat();
	void removeDTux();

	bool bulletReady;

private slots:
	void shoot(QPoint pos);
	void reloadGun();
	void bulletTimeout();

signals:
	void gnatKilled(bool hit);
	void magKilled(bool hit);
	void tuxKilled(bool hit);
	void reload();
	void gameReset(int);
};

#endif
