/***************************************************************************
                          kinsectizid.cpp  -  description
                             -------------------
    begin                : Sam Jul  8 05:02:06 CEST 2000
    copyright            : (C) 2000 by Frank von Daak
    email                : frank.fun@kpage.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qtimer.h>
#include <qlcdnumber.h>
#include <qlabel.h>

#include <kconfig.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <kapp.h>

#include "MyQCanvasView.h"
#include "hiscoredialog.h"
#include "gamefield.h"
#include "gametimer.h"

#include "kinsectizid.h"

#define PLAY_TIME	75
#define MAX_LEVELS	16
#define	MUST_HIT	50

//#define PLAY_TIME	5 // debug
//#define	MUST_HIT	1 // debug

KInsectizid::KInsectizid(QWidget *parent, const char *name, WFlags f) 
			: QWidget(parent, name, f)
{
 gamefield = 0;
 hiscore = 0;
 hiField = 0;
 hiView = 0;

 gameover = false;
 blocked = false;
 hiShow = false;
 hits = 0;
 miss = 0;
 level = 1;
 points = 0;
 MUST_HITS = MUST_HIT;
 paused = false;
 sound = false;
 timer = new QTimer;

 this->setBackgroundColor(black);

 QFont fnt( "helvetica", 24 );
 QColorGroup grp( blue, black, QColor( 0, 255, 0 ), QColor( 64, 64, 64 ),
		black, blue, black );
 QPalette pal( grp, grp, grp );

 countWidg = new QWidget(this);
 countWidg->setBackgroundColor(black);
 countWidg->setGeometry(0, 550, 800, 50);

 QLabel *hitLab = new QLabel(i18n("Hits"), countWidg);
 hitLab->setGeometry(20, 10, 60, 30);
 hitLab->setFont(fnt);
 hitLab->setPalette( pal );

 hitLcd = new QLCDNumber(3, countWidg);
 hitLcd->setFrameStyle( QFrame::NoFrame );
 hitLcd->setGeometry(80, 5, 50, 40);
 hitLcd->setPalette(pal);
 hitLcd->display("-");

 QLabel *missLab = new QLabel(i18n("Missed"), countWidg);
 missLab->setGeometry(150,10,90,30);
 missLab->setFont(fnt);
 missLab->setPalette( pal );

 missLcd = new QLCDNumber(3, countWidg);
 missLcd->setGeometry(240, 5, 50, 40);
 missLcd->setFrameStyle( QFrame::NoFrame );
 missLcd->setPalette(pal);
 missLcd->display("-");

 QLabel *pointLab = new QLabel(i18n("Score :"), countWidg);
 pointLab->setGeometry(290, 10, 80, 30);
 pointLab->setFont(fnt);
 pointLab->setPalette( pal );

 pointLcd = new QLCDNumber(6, countWidg);
 pointLcd->setGeometry(370, 1, 80, 48);
 pointLcd->setFrameStyle( QFrame::NoFrame );
 pointLcd->setPalette(pal);
 pointLcd->display("-");

 QLabel *levLab = new QLabel(i18n("Level"), countWidg);
 levLab->setGeometry(450, 10, 70, 30);
 levLab->setFont(fnt);
 levLab->setPalette( pal );

 levLcd = new QLCDNumber(2, countWidg);
 levLcd->setGeometry(520, 1, 80, 48);
 levLcd->setFrameStyle( QFrame::NoFrame );
 levLcd->setPalette(pal);
 levLcd->display("-");

// The Timer .....
 gameTimer = new GameTimer(this);
 connect(gameTimer, SIGNAL(timeleft(QString)), this, SLOT(displayTime(QString)));
 connect(gameTimer, SIGNAL(timerExit()), this, SLOT(timeOut()));

 QLabel *timerLab = new QLabel(i18n("Time :"), countWidg);
 timerLab->setGeometry(670, 10, 60, 30);
 timerLab->setFont(fnt);
 timerLab->setPalette(pal);

 timerLcd = new QLCDNumber(4, countWidg);
 timerLcd->setGeometry(730, 5, 60, 40);
 timerLcd->setFrameStyle( QFrame::NoFrame );
 timerLcd->setPalette(pal);
 timerLcd->display("-");

// Needed for the cursor
 this->setMouseTracking(true);

// Initialize the GameField
 initGameField(level);

// show the start text
 newGame();
}

KInsectizid::~KInsectizid()
{
//TODO: clean memory leaks; most *should* be done by QT but:
//TEST IT!
}

void KInsectizid::initGameField(int level)
{
// The Gamefield
 removeGameField();
 gamefield = new GameField(this);
 gamefield->setLevel(level);
 gamefield->addBullets(10);

 connect(gamefield, SIGNAL(gnatKilled(bool)), this, SLOT(addPoint(bool)));
 connect(gamefield, SIGNAL(magKilled(bool)), this, SLOT(addMagPoint(bool)));
 connect(gamefield, SIGNAL(tuxKilled(bool)), this, SLOT(addTuxPoint(bool)));
 connect(gamefield, SIGNAL(reload()), this, SLOT(reloadGun()));//unused
 gamefield->show();
}

void KInsectizid::removeGameField()
{
 if (gamefield) {
	delete gamefield;
	gamefield = 0;
 }
}

void KInsectizid::newGame()
{
// just shows the start text
 gamefield->newGame();
 gamefield->addGnats( 4 );
 gamefield->showText(i18n("Let's kill them !!!\n"
		"Press SpaceBar to start the game !\n"
		"You have to kill at least %1 insects, now...").arg(MUST_HITS),
		white);
//showHiscores();
}

void KInsectizid::startGame()
{
// common calls used after newGame() as well as when a new level has begun
// see keyPressEvent()

// Set the Hits, and miss to 0
 missLcd->display(0);
 hitLcd->display(0);
 levLcd->display(level);
 miss = 0;
 hits = 0;
 timer->stop();
 delete timer;
 timer = new QTimer;
}

void KInsectizid::mouseMoveEvent(QMouseEvent *me)
{
 if (me->pos().y() <= 550)
		this->setCursor(crossCursor);
 else
		this->setCursor(arrowCursor);
}

void KInsectizid::addPoint(bool hit)
{
 if ( hit ) {
	hits++;
	hitLcd->display(hits);
	points = points + (20 + level * 5);
 } else {
	miss++;
	missLcd->display(miss);
	if ( points > 0 ) {
		points = points - (20 + level * 5);
		if (points <= 0)
			points = 0;
	}
 }
 pointLcd->display(points);
}

void KInsectizid::addMagPoint(bool hit)
{
 if ( hit ) {
	hits++;
	hitLcd->display(hits);
	points = points+50;
 }
 pointLcd->display(points);
}

void KInsectizid::addTuxPoint(bool hit)
{
 if ( hit ) {
	miss++;
	missLcd->display(miss);
	if ( points > 150 )
		points = points-150;
	else
		points = 0;
 }
 pointLcd->display(points);
}

void KInsectizid::reloadGun()
{
//	bulletWidg->addBullets(10);
}

void KInsectizid::displayTime(QString time)
{
 timerLcd->display(time);
 if ( points > 0 ) {
	points -= 5;
	if (points <= 0)
		points = 0;
	pointLcd->display(points);
 }
}

void KInsectizid::timeOut()
{
 for ( int i = 1; i <= 10 - gamefield->shots; i++) {
	gamefield->removeBullets(0);
 }
	
 if ( hits < MUST_HITS ) {
	gamefield->showText( i18n( "Game Over !!!!" ), yellow );
	gamefield->started = false;
	level = 1;
	blocked = true;
	timer->start(8000);
	connect(timer, SIGNAL(timeout()), this, SLOT(showHiscores()));
	gameover = true;
// Hiscore reached ?
	kapp->config()->setGroup("Hiscores");
	int svHi;
	QString tmp;
	for ( int i=15; i >= 1 ; i--) {
		tmp.sprintf("Score%d",i);
		svHi = kapp->config()->readNumEntry(tmp,0);
		if ( points > svHi ) {
			blTimer = new QTimer();
			blTimer->start(3000);
			connect(blTimer, SIGNAL(timeout()), this, SLOT(unblockIt()));
			break;
		} else {
			points = 0;
			blocked = false;
		}
	}

 } else {
	if ( level <= 2 ) {
		MUST_HITS += 10;
	} else if ( level <= 5 ) {
		MUST_HITS += 5;
	} else {
		MUST_HITS += 2;
	}
	QString txt;
	txt.sprintf(i18n("\t\tT I M E O U T  !!!\n"
			"Press SpaceBar for the next level !\n"
			"You have to kill at least %1 insects, now...").
			arg(MUST_HITS));
	gamefield->showText( txt , yellow );
	gamefield->started = false;
	level++;
 }
}

void KInsectizid::keyPressEvent( QKeyEvent *ke)
{
 removeHiField();

 if ( !gamefield->started && ke->key() == Key_Space && !gameover ) {
//space pressed
	initGameField(level);

	gamefield->newGame();
	gamefield->addGnats( 4 );
	gamefield->started = true;
	gamefield->sound = sound;
	gameTimer->maxTime(PLAY_TIME);
	gameTimer->startTimer(1000);

	startGame();
 } else if ( !gamefield->started && ke->key() == Key_Space && gameover && 
		!blocked ) {
//space pressed when a game is over

	if (hiShow) {
		killTimers();
		hiShow = false;
	}
	
	initGameField(level);
	newGame();
	gameover = false;

	startGame();
 } else if ( ke->key() == Key_P && !paused && gamefield->started ) {
	pause(true);
 } else if ( ke->key() == Key_P && paused && gamefield->started ) {
	pause(false);
 } else if ( ke->key() == Key_Escape) {
	close();//TODO KStdAccel
 } else if ( ke->key() == Key_C ){
	chiefHide(gamefield->started);
 } else if ( ke->key() == Key_S ) {
	toggleSound();
 }
}

void KInsectizid::toggleSound()
{ sound = !sound; gamefield->sound = sound; }

void KInsectizid::chiefHide(bool started)
{
 if ( started ) {
	gamefield->killTimers();
	gameTimer->killTimers();
	gamefield->showPause(true);
	gamefield->view->paused=true;
	paused = true;
 }
 this->setCaption(i18n("KNetwork-Manager"));
 this->iconify();
//TODO: restore caption when the game is shown again
}

void KInsectizid::pause( bool on )
{
 if (on) {
	gamefield->killTimers();
	gameTimer->killTimers();
	gamefield->showPause(true);
	gamefield->view->paused=true;
	paused = true;
 } else {
	gamefield->startTimer(40);
	gameTimer->startTimer(1000);
	gamefield->showPause(false);
	gamefield->view->paused=false;
	paused = false;
 }
}

void KInsectizid::closeEvent(QCloseEvent *ce)
{
 bool userPaused = paused;
 if ( gamefield->started ) {
	gamefield->killTimers();
	gameTimer->killTimers();
	gamefield->view->paused=true;
	paused = true;
 }
 switch (KMessageBox::warningYesNo( this,
		i18n("Do you realy want to exit ?"),
		i18n("Exit Game ?"))) {
	case KMessageBox::Yes:
//		exit(0);//FIXME: use close() or so
		kapp->quit();
		break;
	case KMessageBox::No:
		if ( gamefield->started && !userPaused ) {
			gamefield->startTimer(40);
			gameTimer->startTimer(1000);
			gamefield->showPause(false);
			gamefield->view->paused=false;
			paused = false;
		}
		break;
 }
}

void KInsectizid::showHiscores()
{
 if ( !hiShow ) {
	hiShow = true;
	timer->stop();
	timer->start(15000);

	gamefield->hide();

	hiY = 600;

	hiField = new QCanvas(800, 550);

	hiView = new QCanvasView(hiField, this);
	hiView->setHScrollBarMode(QScrollView::AlwaysOff);
	hiView->setVScrollBarMode(QScrollView::AlwaysOff);
	hiView->setGeometry(0,0,800,550);
	hiField->setBackgroundColor(black);

	hiscore = new QCanvasText(hiField);

	hiscore->setText( i18n("Hiscores ....") );
	hiscore->setFont(QFont("helvetica", 32, QFont::Bold));
	hiscore->setColor( red );
	hiscore->move( (width()-hiscore->boundingRect().width()) / 2, 600 );

//Time to get the HiScores
	QString hiNTxt, hiSTxt, tmp, tmp2;

	hiNTxt = "";
	hiSTxt = "";
	kapp->config()->setGroup("Hiscores");

	for ( int i=1; i <= 15; i++) {
		tmp.sprintf("Player%d",i);
		tmp = kapp->config()->readEntry(tmp,"Empty");
		tmp2.sprintf("%d. ", i);
		hiNTxt = tmp2;
		hiNTxt += tmp;
		hiNames[i] = new QCanvasText(hiField);
		hiNames[i]->setText( hiNTxt );
		hiNames[i]->setFont(QFont("helvetica", 28, QFont::Bold));
		hiNames[i]->setColor(yellow);
		hiNames[i]->move(width() /2 - (hiNames[i]->boundingRect().width() )-30, hiY + 50+ i * 30);
		hiNames[i]->show();
	}

	for ( int i=1; i <= 15 ; i++ ) {
		tmp.sprintf("Score%d",i);
		tmp = kapp->config()->readEntry(tmp, "0");
		hiScores[i] = new QCanvasText(hiField);
		hiScores[i]->setText( tmp );
		hiScores[i]->setFont(QFont("helvetica", 28, QFont::Bold));
		hiScores[i]->setColor(yellow);
		hiScores[i]->move( (width()/2)+200, hiY+50+(i*30) );
		hiScores[i]->show();
	}

	hiscore->show();

	startTimer(10);

	hiView->show();
 } else {
	hiShow = false;
	killTimers();

	removeHiField();
	gamefield->show();
 }
}

void KInsectizid::removeHiField()
{
 if (hiView) {
	for ( int i=1; i <= 15; i++ ){
		delete hiNames[i];
		delete hiScores[i];
		hiNames[i] = 0;
		hiScores[i] = 0;
	}
	delete hiscore;
	delete hiField;
	delete hiView;

	hiscore = 0;
	hiField = 0;
	hiView = 0;
 }
}

void KInsectizid::timerEvent(QTimerEvent *te)
{
 killTimers();
 startTimer(10);
 hiY -= 2;
 hiscore->move( (width()-hiscore->boundingRect().width()) / 2, hiY );
 for ( int i=1; i <= 15; i++ ) {
	hiNames[i]->move( width() /2 - hiNames[i]->boundingRect().width() - 30,
			hiY + 50 + i * 50 );
	hiScores[i]->move( width() /2 + 200, hiY + 50 + i * 50 );
 }
	
 hiField->update();

 if ( hiY <= -850 )
	hiY = 600;
}

void KInsectizid::newHiScore(int pts)
{
 hiScoreDialog = new HiScoreDialog(this);
 hiScoreDialog->setFixedSize(600,400);
 connect(hiScoreDialog, SIGNAL(newName(QString)), this, SLOT(saveNewHi(QString)));
 hiScoreDialog->show();
}

void KInsectizid::saveNewHi(QString name)
{
 QString tmp, tmp2, tmp3;
 QString on[15];
 int op[15];
 int svHi;

 kapp->config()->setGroup("Hiscores");

 for (int i = 0; i < 15; i++) {
	tmp.sprintf("Score%d", i+1);
	svHi = kapp->config()->readNumEntry(tmp, 0);
	if ( points >= svHi ) {
		tmp2.sprintf("Player%d", i+1);
		// Read the old HiScores to buffer
		for ( int o = i; o < 15 ; o++) {
			tmp3.sprintf("Score%d", o+1);
			op[o] = kapp->config()->readNumEntry(tmp3,0);
			tmp3.sprintf("Player%d", o+1);
			on[o] = kapp->config()->readEntry(tmp3, "Empty");
		}
		// Write current HiScore
		kapp->config()->writeEntry(tmp, points);
		kapp->config()->writeEntry(tmp2, name);
		// Write back HiScores from buffer
		for ( int o = i + 1; o < 15 ; o++) {
			tmp3.sprintf("Score%d",o+1);
			kapp->config()->writeEntry(tmp3, op[o]);
			tmp3.sprintf("Player%d",o+1);
			kapp->config()->writeEntry(tmp3, on[o]);
		}
		kapp->config()->sync();
		break;
	}
 }

 delete hiScoreDialog;
}

void KInsectizid::unblockIt()
{
 blTimer->stop();
 blocked = false;
 newHiScore(points);
 points = 0;
}

